/*********************************************//*!@addtogroup file Files*//*@{*/
/*!
 *******************************************************************************
* File identification:      $Id: dialog_Event.qml 46 2016-12-28 21:36:08Z cypax $
* Revision of last commit:  $Rev: 46 $
* Author of last commit:    $Author: cypax $
* Date of last commit:      $Date: 2016-12-28 22:36:08 +0100 (Mi, 28 Dez 2016) $
 *******************************************************************************
 *
 * @author        Cypax
 *
 * @brief         Dialog window for calendar options.
 */
/************************************************************************//*@}*/

import QtQuick 2.4
import QtQuick.Window 2.2
import QtQuick.Controls 1.4
import QtQuick.Layouts 1.1
import QtQuick.Dialogs 1.2
import QtQuick.Controls.Styles 1.3

import QtGraphicalEffects 1.0

import CalendarExampleTypes 1.0


Dialog {
    id: thisDialog
    modality: Qt.WindowModal

    title: "Event options"

    property string evtSummary: ""
    property string evtLocation: ""
    property string evtDescription: ""
    property string evtUID: ""
    property string evtHREF: ""
    property string evtFilename: ""
    property date evtStartDateTime: new Date()
    property date evtEndDateTime: new Date()
    property string evtRrule: ""
    property string evtExdate: ""
    property color calendarColor: "#005500"
    property variant calendarPtr

    function clearOptions() {
        evtSummary = ""
        evtLocation = ""
        evtDescription = ""
        evtUID = ""
        evtHREF = ""
        evtStartDateTime = new Date()
        evtEndDateTime = new Date()
        evtRrule = ""
        evtExdate = ""
    }

    contentItem: Rectangle {
        id: rectContentItem

        implicitWidth: 600
        implicitHeight: 600

        focus: true
        color: calendarColor

        Column {
            id: col_Fields
            spacing: 8
            anchors.fill: parent
            anchors.margins: 8

            Row { width: parent.width; spacing: 8;
                Label { width: 60; color: "#fff"; text: qsTr("Summary:"); }
                TextField { id: tfSummary; width: parent.width-parent.spacing-60; text: evtSummary; }
            }

            Row { width: parent.width; spacing: 8;
                Label { width: 60; color: "#fff"; text: qsTr("Location:"); }
                TextField { id: tfLocation; width: parent.width-parent.spacing-60; text: evtLocation; }
            }

            Row { width: parent.width; spacing: 8;
                Label { width: 60; color: "#fff"; text: qsTr("Description:"); }
                TextArea { id: taDescription; height: 60; width: parent.width-parent.spacing-60; text: evtDescription; }
            }

            Row { width: parent.width; spacing: 8;
                Label { width: 60; color: "#fff"; text: qsTr("UID:"); }
                Label { width: parent.width-parent.spacing-60; color: "#fff"; text: evtUID; }
            }

            Row { width: parent.width; spacing: 8; height: calEndDate.height;
                Calendar {
                    id: calStartDate
                    width: (parent.width - parent.spacing) / 2
                    height: width
                    frameVisible: true
                    weekNumbersVisible: true
                    selectedDate: evtStartDateTime
                }
                Calendar {
                    id: calEndDate
                    width: (parent.width - parent.spacing) / 2
                    height: width
                    frameVisible: true
                    weekNumbersVisible: true
                    selectedDate: evtEndDateTime
                }
            }

            Row { width: parent.width; spacing: 8;
                Row {
                    spacing: 8
                    width: (parent.width - parent.spacing) / 2
                    TextField {
                        id: tfStartTimeHH
                        validator: IntValidator {bottom: 0; top: 23}
                        width: 20
                        text: evtStartDateTime.getHours()
                    }
                    Label { color: "#fff"; text: ":"; }
                    TextField {
                        id: tfStartTimeMM
                        validator: IntValidator {bottom: 0; top: 59}
                        width: 20
                        text: evtStartDateTime.getMinutes()
                    }
                }

                Row {
                    spacing: 8
                    width: (parent.width - parent.spacing) / 2
                    TextField {
                        id: tfEndTimeHH
                        validator: IntValidator {bottom: 0; top: 23}
                        width: 20
                        text: evtEndDateTime.getHours()
                    }
                    Label { color: "#fff"; text: ":"; }
                    TextField {
                        id: tfEndTimeMM
                        validator: IntValidator {bottom: 0; top: 59}
                        width: 20
                        text: evtEndDateTime.getMinutes()
                    }
                }
            }

            Row { width: parent.width; spacing: 8;
                Label { width: 60; color: "#fff"; text: qsTr("Repeats:"); }
                TextField { id: tfRrule; width: parent.width-parent.spacing-60; text: evtRrule; }
            }

            Row { width: parent.width; spacing: 8;
                Label { width: 60; color: "#fff"; text: qsTr("Canceled on:"); }
                TextField { id: tfExdate; width: parent.width-parent.spacing-60; text: evtExdate; }
            }

            Row { width: parent.width; spacing: 8;
                Button {
                    width: (parent.width-parent.spacing*2)/3
                    text: qsTr("Save")
                    onClicked: {

                        if (tfSummary.text !== "") {
                            var sDate = new Date(calStartDate.selectedDate)
                            var eDate = new Date(calEndDate.selectedDate)
                            sDate.setHours(tfStartTimeHH.text)
                            sDate.setMinutes(tfStartTimeMM.text)
                            eDate.setHours(tfEndTimeHH.text)
                            eDate.setMinutes(tfEndTimeMM.text)

                            calendarPtr.saveEvent(evtUID,
                                                  "",
                                                  tfSummary.text,
                                                  tfLocation.text,
                                                  taDescription.text,
                                                  tfRrule.text,
                                                  tfExdate.text,
                                                  sDate,
                                                  eDate)

                            thisDialog.close()
                        } else {
                            msgDlg_MissingData.open()
                        }
                    }
                }

                Button {
                    width: (parent.width-parent.spacing*2)/3
                    text: qsTr("Delete")
                    visible: evtUID !== ""
                    onClicked: {
                        calendarPtr.deleteEvent(evtHREF)
                        thisDialog.close()
                    }
                }

                Button {
                    width: (parent.width-parent.spacing*2)/3
                    text: qsTr("Abort")
                    onClicked: { thisDialog.close() }
                }
            }


        }

        MessageDialog {
            id: msgDlg_MissingData
            title: qsTr("Missing data")
            text: qsTr("Please enter a event summary.")
            icon: StandardIcon.Critical
            standardButtons: StandardButton.Ok
        }

        Keys.onEscapePressed: {
            thisDialog.close()
        }
        Keys.onEnterPressed: {
            thisDialog.close()
        }
        Keys.onReturnPressed: {
            thisDialog.close()
        }

    } //END: contentItem: Column



}
